@@ -141,6 +141,28 @@ enum data_kind {
 const char *
 db_data_kind_label(enum data_kind data_kind);
 
+enum scan_kind {
+  SCAN_KIND_UNKNOWN = 0,
+  SCAN_KIND_FILES = 1,
+  SCAN_KIND_SPOTIFY = 2,
+  SCAN_KIND_RSS = 3,
+};
+
+const char *
+db_scan_kind_label(enum scan_kind scan_kind);
+
+enum scan_kind
+db_scan_kind_enum(const char *label);
+
+/* Indicates user marked status on a track  - values can be bitwise enumerated */
+enum usermark {
+  USERMARK_NA  = 0,
+  USERMARK_DELETE = 1,
+  USERMARK_REXCODE = 2,
+  USERMARK_REVIEW = 4,
+};
+
+
 /* Note that fields marked as integers in the metadata map in filescanner_ffmpeg must be uint32_t here */
 struct media_file_info {
   uint32_t id;
@@ -168,7 +190,7 @@ struct media_file_info {
   uint32_t song_length;
   int64_t file_size;
   uint32_t year;         /* TDRC */
-  uint32_t date_released;
+  int64_t date_released;  // bumped to (signed) int64 since all 32bits are unsigned
 
   uint32_t track;        /* TRCK */
   uint32_t total_tracks;
@@ -198,6 +220,7 @@ struct media_file_info {
   uint32_t time_skipped;
 
   int64_t disabled;      // Long because it stores up to INOTIFY_FAKE_COOKIE
+  uint32_t usermark;     // See enum user_mark { }
 
   uint64_t sample_count; //TODO [unused] sample count is never set and therefor always 0
   char *codectype;       /* song.codectype, 4 chars max (32 bits) */
@@ -223,6 +246,8 @@ struct media_file_info {
   char *album_sort;
   char *album_artist_sort;
   char *composer_sort;
+
+  uint32_t scan_kind; /* Identifies the library_source that created/updates this item */
 };
 
 #define mfi_offsetof(field) offsetof(struct media_file_info, field)
@@ -258,6 +283,7 @@ struct playlist_info {
   uint32_t query_limit;  /* limit, used by e.g. smart playlists */
   uint32_t media_kind;
   char *artwork_url;     /* optional artwork */
+  uint32_t scan_kind; /* Identifies the library_source that created/updates this item */
   uint32_t items;        /* number of items (mimc) */
   uint32_t streams;      /* number of internet streams */
 };
@@ -281,6 +307,7 @@ struct db_playlist_info {
   char *query_limit;
   char *media_kind;
   char *artwork_url;
+  char *scan_kind;
   char *items;
   char *streams;
 };
@@ -393,10 +420,34 @@ struct db_media_file_info {
   char *album_artist_sort;
   char *composer_sort;
   char *channels;
+  char *usermark;
+  char *scan_kind;
 };
 
 #define dbmfi_offsetof(field) offsetof(struct db_media_file_info, field)
 
+/* Info object for generic browse queries that want more info than just
+ * the item string and sort string (e. g. for genre or compose queries
+ * that want to display the total track / album count).
+ */
+struct db_browse_info {
+  char *itemname;
+  char *itemname_sort;
+  char *track_count;
+  char *album_count;
+  char *artist_count;
+  char *song_length;
+  char *data_kind;
+  char *media_kind;
+  char *year;
+  char *date_released;
+  char *time_added;
+  char *time_played;
+  char *seek;
+};
+
+#define dbbi_offsetof(field) offsetof(struct db_browse_info, field)
+
 enum strip_type {
   STRIP_NONE,
   STRIP_PATH,
@@ -423,6 +474,7 @@ struct filecount_info {
   uint64_t length;
   uint32_t artist_count;
   uint32_t album_count;
+  uint64_t file_size;
 };
 
 /* Directory ids must be in sync with the ids in Q_DIR* in db_init.c */
@@ -441,6 +493,7 @@ struct directory_info {
   uint32_t db_timestamp;
   int64_t disabled;
   uint32_t parent_id;
+  uint32_t scan_kind; /* Identifies the library_source that created/updates this item */
 };
 
 struct directory_enum {
@@ -539,6 +592,9 @@ free_pli(struct playlist_info *pli, int content_only);
 void
 free_di(struct directory_info *di, int content_only);
 
+void
+free_wi(struct watch_info *wi, int content_only);
+
 void
 free_query_params(struct query_params *qp, int content_only);
 
@@ -552,6 +608,9 @@ db_hook_post_scan(void);
 void
 db_purge_cruft(time_t ref);
 
+void
+db_purge_cruft_bysource(time_t ref, enum scan_kind scan_kind);
+
 void
 db_purge_all(void);
 
@@ -573,22 +632,25 @@ void
 db_query_end(struct query_params *qp);
 
 int
-db_query_fetch_file(struct query_params *qp, struct db_media_file_info *dbmfi);
+db_query_fetch_file(struct db_media_file_info *dbmfi, struct query_params *qp);
 
 int
-db_query_fetch_pl(struct query_params *qp, struct db_playlist_info *dbpli);
+db_query_fetch_pl(struct db_playlist_info *dbpli, struct query_params *qp);
 
 int
-db_query_fetch_group(struct query_params *qp, struct db_group_info *dbgri);
+db_query_fetch_group(struct db_group_info *dbgri, struct query_params *qp);
 
 int
-db_query_fetch_count(struct query_params *qp, struct filecount_info *fci);
+db_query_fetch_browse(struct db_browse_info *dbbi, struct query_params *qp);
 
 int
-db_query_fetch_string(struct query_params *qp, char **string);
+db_query_fetch_count(struct filecount_info *fci, struct query_params *qp);
 
 int
-db_query_fetch_string_sort(struct query_params *qp, char **string, char **sortstring);
+db_query_fetch_string(char **string, struct query_params *qp);
+
+int
+db_query_fetch_string_sort(char **string, char **sortstring, struct query_params *qp);
 
 /* Files */
 int
@@ -651,6 +713,9 @@ db_file_seek_update(int id, uint32_t seek);
 int
 db_file_rating_update_byid(uint32_t id, uint32_t rating);
 
+int
+db_file_usermark_update_byid(uint32_t id, uint32_t usermark);
+
 int
 db_file_rating_update_byvirtualpath(const char *virtual_path, uint32_t rating);
 
@@ -755,16 +820,19 @@ void
 db_directory_enum_end(struct directory_enum *de);
 
 int
-db_directory_addorupdate(char *virtual_path, char *path, int disabled, int parent_id);
+db_directory_add(struct directory_info *di, int *id);
+
+int
+db_directory_update(struct directory_info *di);
 
 void
 db_directory_ping_bymatch(char *virtual_path);
 
 void
-db_directory_disable_bymatch(char *path, enum strip_type strip, uint32_t cookie);
+db_directory_disable_bymatch(const char *path, enum strip_type strip, uint32_t cookie);
 
 int
-db_directory_enable_bycookie(uint32_t cookie, char *path);
+db_directory_enable_bycookie(uint32_t cookie, const char *path);
 
 int
 db_directory_enable_bypath(char *path);
@@ -920,28 +988,28 @@ int
 db_watch_delete_bywd(uint32_t wd);
 
 int
-db_watch_delete_bypath(char *path);
+db_watch_delete_bypath(const char *path);
 
 int
-db_watch_delete_bymatch(char *path);
+db_watch_delete_bymatch(const char *path);
 
 int
 db_watch_delete_bycookie(uint32_t cookie);
 
 int
-db_watch_get_bywd(struct watch_info *wi);
+db_watch_get_bywd(struct watch_info *wi, int wd);
 
 int
-db_watch_get_bypath(struct watch_info *wi);
+db_watch_get_bypath(struct watch_info *wi, const char *path);
 
 void
-db_watch_mark_bypath(char *path, enum strip_type strip, uint32_t cookie);
+db_watch_mark_bypath(const char *path, enum strip_type strip, uint32_t cookie);
 
 void
-db_watch_mark_bymatch(char *path, enum strip_type strip, uint32_t cookie);
+db_watch_mark_bymatch(const char *path, enum strip_type strip, uint32_t cookie);
 
 void
-db_watch_move_bycookie(uint32_t cookie, char *path);
+db_watch_move_bycookie(uint32_t cookie, const char *path);
 
 int
 db_watch_cookie_known(uint32_t cookie);
