@@ -28,9 +28,6 @@
 #include <limits.h>
 #include <errno.h>
 #include <pthread.h>
-#ifdef HAVE_PTHREAD_NP_H
-# include <pthread_np.h>
-#endif
 #include <time.h>
 #include <sys/param.h>
 #include <sys/queue.h>
@@ -121,7 +118,7 @@ static const struct content_type_map ext2ctype[] =
 
 static const char *http_reply_401 = "<html><head><title>401 Unauthorized</title></head><body>Authorization required</body></html>";
 
-static const char *webroot_directory;
+static char webroot_directory[PATH_MAX];
 struct event_base *evbase_httpd;
 
 #ifdef HAVE_EVENTFD
@@ -886,12 +883,17 @@ httpd_gen_cb(struct evhttp_request *req, void *arg)
 void
 httpd_uri_free(struct httpd_uri_parsed *parsed)
 {
+  int i;
+
   if (!parsed)
     return;
 
   free(parsed->uri_decoded);
   free(parsed->path);
-  free(parsed->path_parts[0]);
+  for (i = 0; i < ARRAY_SIZE(parsed->path_parts); i++)
+    {
+      free(parsed->path_parts[i]);
+    }
 
   evhttp_clear_headers(&(parsed->ev_query));
 
@@ -905,8 +907,9 @@ struct httpd_uri_parsed *
 httpd_uri_parse(const char *uri)
 {
   struct httpd_uri_parsed *parsed;
-  const char *path;
+  char *path = NULL;
   const char *query;
+  char *path_part;
   char *ptr;
   int i;
   int ret;
@@ -940,7 +943,7 @@ httpd_uri_parse(const char *uri)
 	}
     }
 
-  path = evhttp_uri_get_path(parsed->ev_uri);
+  path = strdup(evhttp_uri_get_path(parsed->ev_uri));
   if (!path)
     {
       DPRINTF(E_WARN, L_HTTPD, "No path in request: '%s'\n", parsed->uri);
@@ -954,23 +957,26 @@ httpd_uri_parse(const char *uri)
       goto error;
     }
 
-  CHECK_NULL(L_HTTPD, parsed->path_parts[0] = strdup(parsed->path));
-
-  strtok_r(parsed->path_parts[0], "/", &ptr);
-  for (i = 1; (i < sizeof(parsed->path_parts) / sizeof(parsed->path_parts[0])) && parsed->path_parts[i - 1]; i++)
+  path_part = strtok_r(path, "/", &ptr);
+  for (i = 0; (i < ARRAY_SIZE(parsed->path_parts) && path_part); i++)
     {
-      parsed->path_parts[i] = strtok_r(NULL, "/", &ptr);
+      parsed->path_parts[i] = evhttp_uridecode(path_part, 0, NULL);
+      path_part = strtok_r(NULL, "/", &ptr);
     }
 
-  if (!parsed->path_parts[0] || parsed->path_parts[i - 1] || (i < 2))
+  if (path_part)
     {
-      DPRINTF(E_LOG, L_HTTPD, "URI path has too many/few components (%d): '%s'\n", (parsed->path_parts[0]) ? i : 0, parsed->path);
+      // If "path_part" is not NULL, we have path tokens that could not be parsed into the "parsed->path_parts" array
+      DPRINTF(E_LOG, L_HTTPD, "URI path has too many components (%d): '%s'\n", i, parsed->path);
       goto error;
     }
 
+  free(path);
+
   return parsed;
 
  error:
+  free(path);
   httpd_uri_free(parsed);
   return NULL;
 }
@@ -1000,7 +1006,7 @@ httpd_request_parse(struct evhttp_request *req, struct httpd_uri_parsed *uri_par
 
       evcon = evhttp_request_get_connection(req);
       if (evcon)
-	evhttp_connection_get_peer(evcon, &hreq->peer_address, &hreq->peer_port);
+	httpd_peer_get(&hreq->peer_address, &hreq->peer_port, evcon);
       else
 	DPRINTF(E_LOG, L_HTTPD, "Connection to client lost or missing\n");
 
@@ -1313,9 +1319,10 @@ httpd_stream_file(struct evhttp_request *req, int id)
   if (!transcode)
     {
       /* Hint the OS */
-      posix_fadvise(st->fd, st->start_offset, st->stream_size, POSIX_FADV_WILLNEED);
-      posix_fadvise(st->fd, st->start_offset, st->stream_size, POSIX_FADV_SEQUENTIAL);
-      posix_fadvise(st->fd, st->start_offset, st->stream_size, POSIX_FADV_NOREUSE);
+      if ( (ret = posix_fadvise(st->fd, st->start_offset, st->stream_size, POSIX_FADV_WILLNEED)) != 0 ||
+           (ret = posix_fadvise(st->fd, st->start_offset, st->stream_size, POSIX_FADV_SEQUENTIAL)) != 0 ||
+           (ret = posix_fadvise(st->fd, st->start_offset, st->stream_size, POSIX_FADV_NOREUSE)) != 0 )
+	DPRINTF(E_DBG, L_HTTPD, "posix_fadvise() failed with error %d\n", ret);
     }
 #endif
 
@@ -1356,6 +1363,10 @@ httpd_gzip_deflate(struct evbuffer *in)
   strm.zfree = Z_NULL;
   strm.opaque = Z_NULL;
 
+  // Just to keep Coverity from complaining about uninitialized values
+  strm.total_in = 0;
+  strm.total_out = 0;
+
   // Set up a gzip stream (the "+ 16" in 15 + 16), instead of a zlib stream (default)
   ret = deflateInit2(&strm, Z_DEFAULT_COMPRESSION, Z_DEFLATED, 15 + 16, 8, Z_DEFAULT_STRATEGY);
   if (ret != Z_OK)
@@ -1485,7 +1496,7 @@ bool
 httpd_admin_check_auth(struct evhttp_request *req)
 {
   struct evhttp_connection *evcon;
-  char *addr;
+  const char *addr;
   uint16_t port;
   const char *passwd;
   int ret;
@@ -1497,7 +1508,7 @@ httpd_admin_check_auth(struct evhttp_request *req)
       return false;
     }
 
-  evhttp_connection_get_peer(evcon, &addr, &port);
+  httpd_peer_get(&addr, &port, evcon);
 
   if (net_peer_address_is_trusted(addr))
     return true;
@@ -1626,6 +1637,16 @@ httpd_basic_auth(struct evhttp_request *req, const char *user, const char *passw
   return -1;
 }
 
+void
+httpd_peer_get(const char **address, ev_uint16_t *port, struct evhttp_connection *evcon)
+{
+#ifdef HAVE_EVHTTP_CONNECTION_GET_PEER_CONST_CHAR
+  evhttp_connection_get_peer(evcon, address, port);
+#else
+  evhttp_connection_get_peer(evcon, (char **)address, port);
+#endif
+}
+
 /* Thread: main */
 int
 httpd_init(const char *webroot)
@@ -1649,7 +1670,12 @@ httpd_init(const char *webroot)
 
       return -1;
     }
-  webroot_directory = webroot;
+  if (!realpath(webroot, webroot_directory))
+    {
+      DPRINTF(E_LOG, L_HTTPD, "Web root directory '%s' could not be dereferenced: %s\n", webroot, strerror(errno));
+
+      return -1;
+    }
 
   evbase_httpd = event_base_new();
   if (!evbase_httpd)
@@ -1789,11 +1815,7 @@ httpd_init(const char *webroot)
       goto thread_fail;
     }
 
-#if defined(HAVE_PTHREAD_SETNAME_NP)
-  pthread_setname_np(tid_httpd, "httpd");
-#elif defined(HAVE_PTHREAD_SET_NAME_NP)
-  pthread_set_name_np(tid_httpd, "httpd");
-#endif
+  thread_setname(tid_httpd, "httpd");
 
   return 0;
 
