@@ -46,7 +46,6 @@
 #include "artwork.h"
 #include "dmap_common.h"
 #include "db.h"
-#include "daap_query.h"
 #include "player.h"
 #include "listener.h"
 
@@ -237,7 +236,7 @@ find_first_song_id(const char *query)
   qp.sort = S_NONE;
   qp.offset = 0;
   qp.limit = 1;
-  qp.filter = daap_query_parse_sql(query);
+  qp.filter = dmap_query_parse_sql(query);
   if (!qp.filter)
     {
       DPRINTF(E_LOG, L_DACP, "Improper DAAP query!\n");
@@ -253,7 +252,7 @@ find_first_song_id(const char *query)
       goto no_query_start;
     }
 
-  if (((ret = db_query_fetch_file(&qp, &dbmfi)) == 0) && (dbmfi.id))
+  if ((ret = db_query_fetch_file(&dbmfi, &qp)) == 0)
     {
       ret = safe_atoi32(dbmfi.id, &id);
       if (ret < 0)
@@ -264,7 +263,6 @@ find_first_song_id(const char *query)
 	}
 
       DPRINTF(E_DBG, L_DACP, "Found index song (id %d)\n", id);
-      ret = 1;
     }
   else
     {
@@ -280,7 +278,7 @@ find_first_song_id(const char *query)
   if (qp.filter)
     free(qp.filter);
 
-  if (ret == 1)
+  if (id > 0)
     return id;
   else
     return -1;
@@ -379,7 +377,7 @@ dacp_queueitem_add(const char *query, const char *queuefilter, const char *sort,
 
 	      return -1;
 	    }
-	  qp.filter = daap_query_parse_sql(buf);
+	  qp.filter = dmap_query_parse_sql(buf);
 	}
       else
 	{
@@ -388,14 +386,14 @@ dacp_queueitem_add(const char *query, const char *queuefilter, const char *sort,
 	  // If the queuefilter is unkown, ignore it and use the query parameter instead to build the sql query
 	  id = 0;
 	  qp.type = Q_ITEMS;
-	  qp.filter = daap_query_parse_sql(query);
+	  qp.filter = dmap_query_parse_sql(query);
 	}
     }
   else
     {
       id = 0;
       qp.type = Q_ITEMS;
-      qp.filter = daap_query_parse_sql(query);
+      qp.filter = dmap_query_parse_sql(query);
     }
 
   if (sort)
@@ -787,10 +785,6 @@ dacp_playstatus_update_handler(short event_mask)
 {
   int ret;
 
-  // Only send status update on player change events
-  if (!(event_mask & (LISTENER_PLAYER | LISTENER_VOLUME)))
-    return;
-
 #ifdef HAVE_EVENTFD
   ret = eventfd_write(update_efd, 1);
   if (ret < 0)
@@ -1002,7 +996,7 @@ dacp_propset_devicevolume(const char *value, struct httpd_request *hreq)
   if (speaker_get(&speaker_info, hreq, "device-volume") < 0)
     return;
 
-  player_volume_update_speaker(speaker_info.id, value);
+  player_volume_setraw_speaker(speaker_info.id, value);
 }
 
 // See player.c:speaker_prevent_playback_set() for comments regarding
@@ -1302,10 +1296,10 @@ dacp_reply_cue_play(struct httpd_request *hreq)
       else
 	{
 	  /* Play from Up Next queue */
-	  if (status.status == PLAY_STOPPED && pos > 0)
-	    pos--;
-
-	  queue_item = db_queue_fetch_byposrelativetoitem(pos, status.item_id, status.shuffle);
+	  if (status.status == PLAY_STOPPED)
+            queue_item = db_queue_fetch_bypos(pos, status.shuffle);
+          else
+            queue_item = db_queue_fetch_byposrelativetoitem(pos, status.item_id, status.shuffle);
 	  if (!queue_item)
 	    {
 	      DPRINTF(E_LOG, L_DACP, "Could not fetch item from queue: pos=%d, now playing=%d\n", pos, status.item_id);
@@ -1411,7 +1405,12 @@ dacp_reply_play(struct httpd_request *hreq)
   if (ret < 0)
     return -1;
 
-  player_playback_start();
+  ret = player_playback_start();
+  if (ret < 0)
+    {
+      httpd_send_error(hreq->req, 500, "Internal Server Error");
+      return -1;
+    }
 
   /* 204 No Content is the canonical reply */
   httpd_send_reply(hreq->req, HTTP_NOCONTENT, "No Content", hreq->reply, HTTPD_SEND_NO_GZIP);
@@ -2732,10 +2731,12 @@ dacp_reply_mutetoggle(struct httpd_request *hreq)
     }
 
   // We don't actually mute, because the player doesn't currently support unmuting
-  if (speaker_info.selected)
-    player_speaker_disable(speaker_info.id);
-  else
-    player_speaker_enable(speaker_info.id);
+  ret = speaker_info.selected ? player_speaker_disable(speaker_info.id) : player_speaker_enable(speaker_info.id);
+  if (ret < 0)
+    {
+      httpd_send_error(hreq->req, 500, "Internal Server Error");
+      return -1;
+    }
 
   /* 204 No Content is the canonical reply */
   httpd_send_reply(hreq->req, HTTP_NOCONTENT, "No Content", hreq->reply, HTTPD_SEND_NO_GZIP);
@@ -2968,7 +2969,7 @@ dacp_init(void)
       return -1;
     }
 
-  listener_add(dacp_playstatus_update_handler, LISTENER_PLAYER | LISTENER_VOLUME);
+  listener_add(dacp_playstatus_update_handler, LISTENER_PLAYER | LISTENER_VOLUME | LISTENER_QUEUE);
 
   return 0;
 
