@@ -97,7 +97,7 @@ static int streaming_meta[2];
 #define STREAMING_ICY_METAINT_DEFAULT  16384
 static unsigned short streaming_icy_metaint = STREAMING_ICY_METAINT_DEFAULT;
 static unsigned streaming_icy_clients;
-static char *streaming_icy_title;
+static char streaming_icy_title[STREAMING_ICY_METATITLELEN_MAX];
 
 
 static void
@@ -106,12 +106,12 @@ streaming_close_cb(struct evhttp_connection *evcon, void *arg)
   struct streaming_session *this;
   struct streaming_session *session;
   struct streaming_session *prev;
-  char *address;
+  const char *address;
   ev_uint16_t port;
 
   this = (struct streaming_session *)arg;
 
-  evhttp_connection_get_peer(evcon, &address, &port);
+  httpd_peer_get(&address, &port, evcon);
   DPRINTF(E_INFO, L_STREAMING, "Stopping mp3 streaming to %s:%d\n", address, (int)port);
 
   pthread_mutex_lock(&streaming_sessions_lck);
@@ -168,7 +168,7 @@ streaming_end(void)
 {
   struct streaming_session *session;
   struct evhttp_connection *evcon;
-  char *address;
+  const char *address;
   ev_uint16_t port;
 
   pthread_mutex_lock(&streaming_sessions_lck);
@@ -178,7 +178,7 @@ streaming_end(void)
       if (evcon)
 	{
 	  evhttp_connection_set_closecb(evcon, NULL, NULL);
-	  evhttp_connection_get_peer(evcon, &address, &port);
+	  httpd_peer_get(&address, &port, evcon);
 	  DPRINTF(E_INFO, L_STREAMING, "Force close stream to %s:%d\n", address, (int)port);
 	}
       evhttp_send_reply_end(session->req);
@@ -283,8 +283,8 @@ encode_buffer(uint8_t *buffer, size_t size)
 static uint8_t *
 streaming_icy_meta_create(uint8_t buf[STREAMING_ICY_METALEN_MAX+1], const char *title, unsigned *buflen)
 {
-  unsigned titlelen = 0;
-  unsigned metalen = 0;
+  unsigned titlelen;
+  unsigned metalen;
   uint8_t no16s;
 
   *buflen = 0;
@@ -346,38 +346,28 @@ streaming_icy_meta_splice(const uint8_t *data, size_t datalen, off_t offset, siz
 }
 
 static void
-streaming_player_status_update()
+streaming_player_status_update(void)
 {
-  unsigned x, y;
-  struct db_queue_item *queue_item = NULL;
-  struct player_status  tmp;
+  struct db_queue_item *queue_item;
+  uint32_t prev_id;
 
-  tmp.id = streaming_player_status.id;
+  prev_id = streaming_player_status.id;
   player_get_status(&streaming_player_status);
 
-  if (tmp.id != streaming_player_status.id && streaming_icy_clients)
+  if (prev_id == streaming_player_status.id || !streaming_icy_clients)
     {
-      free(streaming_icy_title);
-      if ( (queue_item = db_queue_fetch_byfileid(streaming_player_status.id)) == NULL)
-	{
-	  streaming_icy_title = NULL;
-	}
-      else
-	{
-	  x = strlen(queue_item->title);
-	  y = strlen(queue_item->artist);
-	  if (x && y)
-	    {
-	      streaming_icy_title = malloc(x+y+4);
-	      snprintf(streaming_icy_title, x+y+4, "%s - %s", queue_item->title, queue_item->artist);
-	    }
-	  else
-	    {
-	      streaming_icy_title = strdup( x ? queue_item->title : queue_item->artist);
-	    }
-	  free_queue_item(queue_item, 0);
-	}
+      return;
     }
+
+  queue_item = db_queue_fetch_byfileid(streaming_player_status.id);
+  if (!queue_item)
+    {
+      streaming_icy_title[0] = '\0';
+      return;
+    }
+
+  snprintf(streaming_icy_title, sizeof(streaming_icy_title), "%s - %s", queue_item->title, queue_item->artist);
+  free_queue_item(queue_item, 0);
 }
 
 static void
@@ -541,7 +531,7 @@ streaming_request(struct evhttp_request *req, struct httpd_uri_parsed *uri_parse
   struct evkeyvalq *output_headers;
   cfg_t *lib;
   const char *name;
-  char *address;
+  const char *address;
   ev_uint16_t port;
   const char *param;
   bool require_icy = false;
@@ -556,7 +546,7 @@ streaming_request(struct evhttp_request *req, struct httpd_uri_parsed *uri_parse
     }
 
   evcon = evhttp_request_get_connection(req);
-  evhttp_connection_get_peer(evcon, &address, &port);
+  httpd_peer_get(&address, &port, evcon);
   param = evhttp_find_header( evhttp_request_get_input_headers(req), "Icy-MetaData");
   if (param && strcmp(param, "1") == 0)
     require_icy = true;
@@ -656,6 +646,7 @@ streaming_init(void)
     default:
       DPRINTF(E_LOG, L_STREAMING, "Unsuppported streaming bit_rate=%d, supports: 64/96/128/192/320, defaulting\n", val);
   }
+
   DPRINTF(E_INFO, L_STREAMING, "Streaming quality: %d/%d/%d @ %dkbps\n", streaming_quality_out.sample_rate, streaming_quality_out.bits_per_sample, streaming_quality_out.channels, streaming_quality_out.bit_rate/1000);
 
   val = cfg_getint(cfgsec, "icy_metaint");
@@ -665,7 +656,12 @@ streaming_init(void)
   else
     DPRINTF(E_INFO, L_STREAMING, "Unsupported icy_metaint=%d, supported range: 4096..131072, defaulting to %d\n", val, streaming_icy_metaint);
 
-  pthread_mutex_init(&streaming_sessions_lck, NULL);
+  ret = mutex_init(&streaming_sessions_lck);
+  if (ret < 0)
+    {
+      DPRINTF(E_FATAL, L_STREAMING, "Could not initialize mutex (%d): %s\n", ret, strerror(ret));
+      goto error;
+    }
 
   // Non-blocking because otherwise httpd and player thread may deadlock
 #ifdef HAVE_PIPE2
@@ -715,7 +711,6 @@ streaming_init(void)
   CHECK_NULL(L_STREAMING, metaev = event_new(evbase_httpd, streaming_meta[0], EV_READ | EV_PERSIST, streaming_meta_cb, NULL));
 
   streaming_icy_clients = 0;
-  streaming_icy_title = NULL;
 
   return 0;
 
@@ -746,7 +741,6 @@ streaming_deinit(void)
 
   transcode_encode_cleanup(&streaming_encode_ctx);
   evbuffer_free(streaming_encoded_data);
-  free(streaming_icy_title);
 
   pthread_mutex_destroy(&streaming_sessions_lck);
 }
