@@ -26,15 +26,9 @@
 #include <fcntl.h>
 #include <limits.h>
 #include <pthread.h>
-#ifdef HAVE_PTHREAD_NP_H
-# include <pthread_np.h>
-#endif
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unictype.h>
-#include <uninorm.h>
-#include <unistr.h>
 
 #include <event2/event.h>
 
@@ -79,14 +73,14 @@ static pthread_t tid_library;
 struct event_base *evbase_lib;
 
 extern struct library_source filescanner;
-#ifdef HAVE_SPOTIFY_H
+#ifdef SPOTIFY
 extern struct library_source spotifyscanner;
 #endif
 extern struct library_source rssscanner;
 
 static struct library_source *sources[] = {
     &filescanner,
-#ifdef HAVE_SPOTIFY_H
+#ifdef SPOTIFY
     &spotifyscanner,
 #endif
     &rssscanner,
@@ -125,10 +119,10 @@ static struct library_callback_register library_cb_register[LIBRARY_MAX_CALLBACK
 int
 library_media_save(struct media_file_info *mfi)
 {
-  if (!mfi->path || !mfi->fname)
+  if (!mfi->path || !mfi->fname || !mfi->scan_kind)
     {
-      DPRINTF(E_LOG, L_LIB, "Ignoring media file with missing values (path='%s', fname='%s', data_kind='%d')\n",
-	      mfi->path, mfi->fname, mfi->data_kind);
+      DPRINTF(E_LOG, L_LIB, "Ignoring media file with missing values (path='%s', fname='%s', scan_kind='%d', data_kind='%d')\n",
+	      mfi->path, mfi->fname, mfi->scan_kind, mfi->data_kind);
       return -1;
     }
 
@@ -148,9 +142,10 @@ library_media_save(struct media_file_info *mfi)
 int
 library_playlist_save(struct playlist_info *pli)
 {
-  if (!pli->path)
+  if (!pli->path || !pli->scan_kind)
     {
-      DPRINTF(E_LOG, L_LIB, "Ignoring playlist file with missing path\n");
+      DPRINTF(E_LOG, L_LIB, "Ignoring playlist with missing values (path='%s', scan_kind='%d')\n",
+	      pli->path, pli->scan_kind);
       return -1;
     }
 
@@ -167,6 +162,39 @@ library_playlist_save(struct playlist_info *pli)
     return db_pl_update(pli);
 }
 
+int
+library_directory_save(char *virtual_path, char *path, int disabled, int parent_id, enum scan_kind scan_kind)
+{
+  struct directory_info di = { 0 };
+  int id;
+  int ret;
+
+  id = db_directory_id_byvirtualpath(virtual_path);
+
+  di.id = id;
+  di.parent_id = parent_id;
+  di.virtual_path = safe_strdup(virtual_path);
+  di.path = safe_strdup(path);
+  di.disabled = disabled;
+  di.db_timestamp = (uint64_t)time(NULL);
+  di.scan_kind = scan_kind;
+
+  if (di.id == 0)
+    ret = db_directory_add(&di, &id);
+  else
+    ret = db_directory_update(&di);
+
+  free_di(&di, 1);
+
+  if (ret < 0 || id <= 0)
+  {
+    DPRINTF(E_LOG, L_DB, "Insert or update of directory failed '%s'\n", virtual_path);
+    return -1;
+  }
+
+  return id;
+}
+
 static void
 scheduled_cb(int fd, short what, void *arg)
 {
@@ -233,7 +261,7 @@ library_callback_schedule(library_cb cb, void *arg, struct timeval *wait, enum l
 
   event_add(cbreg->ev, wait);
 
-  DPRINTF(E_DBG, L_LIB, "Added library callback to %p (id %d), wait %ld.%06ld\n", cbreg->cb, idx_available, wait->tv_sec, wait->tv_usec);
+  DPRINTF(E_DBG, L_LIB, "Added library callback to %p (id %d), wait %ld.%06ld\n", cbreg->cb, idx_available, (long) wait->tv_sec, (long) wait->tv_usec);
 
   return idx_available;
 }
@@ -262,20 +290,27 @@ handle_deferred_update_notifications(void)
 }
 
 static void
-purge_cruft(time_t start)
+purge_cruft(time_t start, enum scan_kind scan_kind)
 {
   DPRINTF(E_DBG, L_LIB, "Purging old library content\n");
-  db_purge_cruft(start);
+  if (scan_kind > 0)
+    db_purge_cruft_bysource(start, scan_kind);
+  else
+    db_purge_cruft(start);
   db_groups_cleanup();
   db_queue_cleanup();
 
-  DPRINTF(E_DBG, L_LIB, "Purging old artwork content\n");
-  cache_artwork_purge_cruft(start);
+  if (scan_kind <= 0)
+    {
+      DPRINTF(E_DBG, L_LIB, "Purging old artwork content\n");
+      cache_artwork_purge_cruft(start);
+    }
 }
 
 static enum command_state
 rescan(void *arg, int *ret)
 {
+  enum scan_kind *scan_kind;
   time_t starttime;
   time_t endtime;
   int i;
@@ -284,20 +319,29 @@ rescan(void *arg, int *ret)
   listener_notify(LISTENER_UPDATE);
   starttime = time(NULL);
 
+  scan_kind = arg;
+
   for (i = 0; sources[i]; i++)
     {
       if (!sources[i]->disabled && sources[i]->rescan)
 	{
-	  DPRINTF(E_INFO, L_LIB, "Rescan library source '%s'\n", sources[i]->name);
-	  sources[i]->rescan();
+	  if (*scan_kind > 0 && *scan_kind != sources[i]->scan_kind)
+	    {
+	      DPRINTF(E_DBG, L_LIB, "Skipping library source '%s'\n", db_scan_kind_label(sources[i]->scan_kind));
+	    }
+	  else
+	    {
+	      DPRINTF(E_INFO, L_LIB, "Rescan library source '%s'\n", db_scan_kind_label(sources[i]->scan_kind));
+	      sources[i]->rescan();
+	    }
 	}
       else
 	{
-	  DPRINTF(E_INFO, L_LIB, "Library source '%s' is disabled\n", sources[i]->name);
+	  DPRINTF(E_INFO, L_LIB, "Library source '%s' is disabled\n", db_scan_kind_label(sources[i]->scan_kind));
 	}
     }
 
-  purge_cruft(starttime);
+  purge_cruft(starttime, *scan_kind);
 
   DPRINTF(E_DBG, L_LIB, "Running post library scan jobs\n");
   db_hook_post_scan();
@@ -318,6 +362,7 @@ rescan(void *arg, int *ret)
 static enum command_state
 metarescan(void *arg, int *ret)
 {
+  enum scan_kind *scan_kind;
   time_t starttime;
   time_t endtime;
   int i;
@@ -326,20 +371,29 @@ metarescan(void *arg, int *ret)
   listener_notify(LISTENER_UPDATE);
   starttime = time(NULL);
 
+  scan_kind = arg;
+
   for (i = 0; sources[i]; i++)
     {
       if (!sources[i]->disabled && sources[i]->metarescan)
 	{
-	  DPRINTF(E_INFO, L_LIB, "Meta rescan library source '%s'\n", sources[i]->name);
-	  sources[i]->metarescan();
+	  if (*scan_kind > 0 && *scan_kind != sources[i]->scan_kind)
+	    {
+	      DPRINTF(E_DBG, L_LIB, "Skipping library source '%s'\n", db_scan_kind_label(sources[i]->scan_kind));
+	    }
+	  else
+	    {
+	      DPRINTF(E_INFO, L_LIB, "Meta rescan library source '%s'\n", db_scan_kind_label(sources[i]->scan_kind));
+	      sources[i]->metarescan();
+	    }
 	}
       else
 	{
-	  DPRINTF(E_INFO, L_LIB, "Library source '%s' is disabled\n", sources[i]->name);
+	  DPRINTF(E_INFO, L_LIB, "Library source '%s' is disabled\n", db_scan_kind_label(sources[i]->scan_kind));
 	}
     }
 
-  purge_cruft(starttime);
+  purge_cruft(starttime, *scan_kind);
 
   DPRINTF(E_DBG, L_LIB, "Running post library scan jobs\n");
   db_hook_post_scan();
@@ -377,12 +431,12 @@ fullrescan(void *arg, int *ret)
     {
       if (!sources[i]->disabled && sources[i]->fullrescan)
 	{
-	  DPRINTF(E_INFO, L_LIB, "Full-rescan library source '%s'\n", sources[i]->name);
+	  DPRINTF(E_INFO, L_LIB, "Full-rescan library source '%s'\n", db_scan_kind_label(sources[i]->scan_kind));
 	  sources[i]->fullrescan();
 	}
       else
 	{
-	  DPRINTF(E_INFO, L_LIB, "Library source '%s' is disabled\n", sources[i]->name);
+	  DPRINTF(E_INFO, L_LIB, "Library source '%s' is disabled\n", db_scan_kind_label(sources[i]->scan_kind));
 	}
     }
 
@@ -412,7 +466,7 @@ playlist_item_add(void *arg, int *retval)
     {
       if (sources[i]->disabled || !sources[i]->playlist_item_add)
 	{
-	  DPRINTF(E_DBG, L_LIB, "Library source '%s' is disabled or does not support playlist_item_add\n", sources[i]->name);
+	  DPRINTF(E_DBG, L_LIB, "Library source '%s' is disabled or does not support playlist_item_add\n", db_scan_kind_label(sources[i]->scan_kind));
 	  continue;
 	}
 
@@ -420,7 +474,7 @@ playlist_item_add(void *arg, int *retval)
 
       if (ret == LIBRARY_OK)
 	{
-	  DPRINTF(E_DBG, L_LIB, "Adding item '%s' to playlist '%s' with library source '%s'\n", param->vp_item, param->vp_playlist, sources[i]->name);
+	  DPRINTF(E_DBG, L_LIB, "Adding item '%s' to playlist '%s' with library source '%s'\n", param->vp_item, param->vp_playlist, db_scan_kind_label(sources[i]->scan_kind));
 	  listener_notify(LISTENER_STORED_PLAYLIST);
 	  break;
 	}
@@ -443,7 +497,7 @@ playlist_remove(void *arg, int *retval)
     {
       if (sources[i]->disabled || !sources[i]->playlist_remove)
 	{
-	  DPRINTF(E_DBG, L_LIB, "Library source '%s' is disabled or does not support playlist_remove\n", sources[i]->name);
+	  DPRINTF(E_DBG, L_LIB, "Library source '%s' is disabled or does not support playlist_remove\n", db_scan_kind_label(sources[i]->scan_kind));
 	  continue;
 	}
 
@@ -451,7 +505,7 @@ playlist_remove(void *arg, int *retval)
 
       if (ret == LIBRARY_OK)
 	{
-	  DPRINTF(E_DBG, L_LIB, "Removing playlist '%s' with library source '%s'\n", virtual_path, sources[i]->name);
+	  DPRINTF(E_DBG, L_LIB, "Removing playlist '%s' with library source '%s'\n", virtual_path, db_scan_kind_label(sources[i]->scan_kind));
 	  listener_notify(LISTENER_STORED_PLAYLIST);
 	  break;
 	}
@@ -475,7 +529,7 @@ queue_item_add(void *arg, int *retval)
     {
       if (sources[i]->disabled || !sources[i]->queue_item_add)
         {
-	  DPRINTF(E_DBG, L_LIB, "Library source '%s' is disabled or does not support queue_add\n", sources[i]->name);
+	  DPRINTF(E_DBG, L_LIB, "Library source '%s' is disabled or does not support queue_add\n", db_scan_kind_label(sources[i]->scan_kind));
 	  continue;
 	}
 
@@ -483,7 +537,7 @@ queue_item_add(void *arg, int *retval)
 
       if (ret == LIBRARY_OK)
 	{
-	  DPRINTF(E_DBG, L_LIB, "Items for path '%s' from library source '%s' added to the queue\n", param->path, sources[i]->name);
+	  DPRINTF(E_DBG, L_LIB, "Items for path '%s' from library source '%s' added to the queue\n", param->path, db_scan_kind_label(sources[i]->scan_kind));
 	  break;
 	}
     }
@@ -508,7 +562,7 @@ queue_save(void *arg, int *retval)
     {
       if (sources[i]->disabled || !sources[i]->queue_save)
 	{
-	  DPRINTF(E_DBG, L_LIB, "Library source '%s' is disabled or does not support queue_save\n", sources[i]->name);
+	  DPRINTF(E_DBG, L_LIB, "Library source '%s' is disabled or does not support queue_save\n", db_scan_kind_label(sources[i]->scan_kind));
 	  continue;
 	}
 
@@ -516,7 +570,7 @@ queue_save(void *arg, int *retval)
 
       if (ret == LIBRARY_OK)
 	{
-	  DPRINTF(E_DBG, L_LIB, "Saving queue to path '%s' with library source '%s'\n", virtual_path, sources[i]->name);
+	  DPRINTF(E_DBG, L_LIB, "Saving queue to path '%s' with library source '%s'\n", virtual_path, db_scan_kind_label(sources[i]->scan_kind));
 	  listener_notify(LISTENER_STORED_PLAYLIST);
 	  break;
 	}
@@ -539,7 +593,7 @@ item_add(void *arg, int *retval)
     {
       if (sources[i]->disabled || !sources[i]->item_add)
 	{
-	  DPRINTF(E_DBG, L_LIB, "Library source '%s' is disabled or does not support add_item\n", sources[i]->name);
+	  DPRINTF(E_DBG, L_LIB, "Library source '%s' is disabled or does not support add_item\n", db_scan_kind_label(sources[i]->scan_kind));
 	  continue;
 	}
 
@@ -547,7 +601,7 @@ item_add(void *arg, int *retval)
 
       if (ret == LIBRARY_OK)
 	{
-	  DPRINTF(E_DBG, L_LIB, "Add item to path '%s' with library source '%s'\n", path, sources[i]->name);
+	  DPRINTF(E_DBG, L_LIB, "Add item to path '%s' with library source '%s'\n", path, db_scan_kind_label(sources[i]->scan_kind));
 	  listener_notify(LISTENER_DATABASE);
 	  break;
 	}
@@ -600,29 +654,39 @@ update_trigger(void *arg, int *retval)
 /* ----------------------- LIBRARY EXTERNAL INTERFACE ---------------------- */
 
 void
-library_rescan()
+library_rescan(enum scan_kind scan_kind)
 {
+  int *param;
+
   if (scanning)
     {
       DPRINTF(E_INFO, L_LIB, "Scan already running, ignoring request to trigger a new init scan\n");
       return;
     }
 
-  scanning = true; // TODO Guard "scanning" with a mutex
-  commands_exec_async(cmdbase, rescan, NULL);
+  scanning = true;
+  param = malloc(sizeof(int));
+  *param = scan_kind;
+
+  commands_exec_async(cmdbase, rescan, param);
 }
 
 void
-library_metarescan()
+library_metarescan(enum scan_kind scan_kind)
 {
+  int *param;
+
   if (scanning)
     {
       DPRINTF(E_INFO, L_LIB, "Scan already running, ignoring request to trigger metadata scan\n");
       return;
     }
 
-  scanning = true; // TODO Guard "scanning" with a mutex
-  commands_exec_async(cmdbase, metarescan, NULL);
+  scanning = true;
+  param = malloc(sizeof(int));
+  *param = scan_kind;
+
+  commands_exec_async(cmdbase, metarescan, param);
 }
 
 void
@@ -634,7 +698,7 @@ library_fullrescan()
       return;
     }
 
-  scanning = true; // TODO Guard "scanning" with a mutex
+  scanning = true;
   commands_exec_async(cmdbase, fullrescan, NULL);
 }
 
@@ -651,7 +715,15 @@ initscan()
   listener_notify(LISTENER_UPDATE);
 
   // Only clear the queue if enabled (default) in config
-  clear_queue_disabled = cfg_getbool(cfg_getsec(cfg, "mpd"), "clear_queue_on_stop_disable");
+  clear_queue_disabled = cfg_getbool(cfg_getsec(cfg, "library"), "clear_queue_on_stop_disable");
+
+  /* Handle deprecated config options */
+  if (0 < cfg_opt_size(cfg_getopt(cfg_getsec(cfg, "mpd"), "clear_queue_on_stop_disable")))
+    {
+      DPRINTF(E_LOG, L_MPD, "Found deprecated option 'clear_queue_on_stop_disable' in section 'mpd', please update configuration file (move option to section 'library').\n");
+      clear_queue_disabled = cfg_getbool(cfg_getsec(cfg, "mpd"), "clear_queue_on_stop_disable");
+    }
+
   if (!clear_queue_disabled)
     {
       db_queue_clear(0);
@@ -665,7 +737,7 @@ initscan()
 
   if (! (cfg_getbool(cfg_getsec(cfg, "library"), "filescan_disable")))
     {
-      purge_cruft(starttime);
+      purge_cruft(starttime, 0);
 
       DPRINTF(E_DBG, L_LIB, "Running post library scan jobs\n");
       db_hook_post_scan();
@@ -797,11 +869,17 @@ library_item_add(const char *path)
       return -1;
     }
 
-  scanning = true; // TODO Guard "scanning" with a mutex
+  scanning = true;
 
   return commands_exec_sync(cmdbase, item_add, NULL, (char *)path);
 }
 
+struct library_source **
+library_sources(void)
+{
+  return sources;
+}
+
 int
 library_exec_async(command_function func, void *arg)
 {
@@ -866,7 +944,7 @@ library_init(void)
     {
       if (!sources[i]->initscan || !sources[i]->rescan || !sources[i]->metarescan || !sources[i]->fullrescan)
 	{
-	  DPRINTF(E_FATAL, L_LIB, "BUG: library source '%s' is missing a scanning method\n", sources[i]->name);
+	  DPRINTF(E_FATAL, L_LIB, "BUG: library source '%s' is missing a scanning method\n", db_scan_kind_label(sources[i]->scan_kind));
 	  return -1;
 	}
 
@@ -882,11 +960,7 @@ library_init(void)
 
   CHECK_ERR(L_LIB, pthread_create(&tid_library, NULL, library, NULL));
 
-#if defined(HAVE_PTHREAD_SETNAME_NP)
-  pthread_setname_np(tid_library, "library");
-#elif defined(HAVE_PTHREAD_SET_NAME_NP)
-  pthread_set_name_np(tid_library, "library");
-#endif
+  thread_setname(tid_library, "library");
 
   return 0;
 }
