@@ -35,6 +35,7 @@
 #include "logger.h"
 #include "misc.h"
 #include "http.h"
+#include "conffile.h"
 
 /* Mapping between the metadata name(s) and the offset
  * of the equivalent metadata field in struct media_file_info */
@@ -55,6 +56,27 @@ err2str(int errnum)
   return errbuf;
 }
 
+static int
+parse_genre(struct media_file_info *mfi, char *genre_string)
+{
+  char **genre = (char**)((char *) mfi + mfi_offsetof(genre));
+  char *ptr;
+
+  if (*genre) // Previous genre tag exists
+    return 0;
+
+  *genre = strdup(genre_string);
+
+  if (cfg_getbool(cfg_getsec(cfg, "library"), "only_first_genre"))
+    {
+      ptr = strchr(*genre, ';');
+      if (ptr)
+        *ptr = '\0';
+    }
+
+  return 1;
+}
+
 static int
 parse_slash_separated_ints(char *string, uint32_t *firstval, uint32_t *secondval)
 {
@@ -98,7 +120,8 @@ parse_date(struct media_file_info *mfi, char *date_string)
 {
   char year_string[32];
   uint32_t *year = (uint32_t *) ((char *) mfi + mfi_offsetof(year));
-  uint32_t *date_released = (uint32_t *) ((char *) mfi + mfi_offsetof(date_released));
+  // signed in db.h to handle dates before 1970
+  int64_t *date_released = (int64_t *) ((char *) mfi + mfi_offsetof(date_released));
   struct tm tm = { 0 };
   int ret = 0;
 
@@ -111,7 +134,7 @@ parse_date(struct media_file_info *mfi, char *date_string)
        || strptime(date_string, "%F", &tm)
      )
     {
-      *date_released = (uint32_t)mktime(&tm);
+      *date_released = mktime(&tm);
       ret++;
     }
 
@@ -120,7 +143,7 @@ parse_date(struct media_file_info *mfi, char *date_string)
       snprintf(year_string, sizeof(year_string), "%" PRIu32 "-01-01T12:00:00", *year);
       if (strptime(year_string, "%FT%T", &tm))
 	{
-	  *date_released = (uint32_t)mktime(&tm);
+	  *date_released = mktime(&tm);
 	  ret++;
 	}
     }
@@ -148,7 +171,7 @@ static const struct metadata_map md_map_generic[] =
     { "author",       0, mfi_offsetof(artist),             NULL },
     { "album_artist", 0, mfi_offsetof(album_artist),       NULL },
     { "album",        0, mfi_offsetof(album),              NULL },
-    { "genre",        0, mfi_offsetof(genre),              NULL },
+    { "genre",        0, mfi_offsetof(genre),              parse_genre },
     { "composer",     0, mfi_offsetof(composer),           NULL },
     { "grouping",     0, mfi_offsetof(grouping),           NULL },
     { "orchestra",    0, mfi_offsetof(orchestra),          NULL },
@@ -400,6 +423,12 @@ scan_metadata_ffmpeg(struct media_file_info *mfi, const char *file)
 
       av_dict_set(&options, "icy", "1", 0);
     }
+  else if (mfi->data_kind == DATA_KIND_FILE && mfi->file_size == 0)
+    {
+      // a 0-byte mp3 will make ffmpeg die with arithmetic exception (with 3.2.15-0+deb9u4)
+      free(path);
+      return -1;
+    }
 
   ret = avformat_open_input(&ctx, path, NULL, &options);
 
@@ -446,7 +475,12 @@ scan_metadata_ffmpeg(struct media_file_info *mfi, const char *file)
       codec_id = ctx->streams[i]->codecpar->codec_id;
       sample_rate = ctx->streams[i]->codecpar->sample_rate;
       sample_fmt = ctx->streams[i]->codecpar->format;
+// Matches USE_CH_LAYOUT in transcode.c
+#if (LIBAVCODEC_VERSION_MAJOR > 59) || ((LIBAVCODEC_VERSION_MAJOR == 59) && (LIBAVCODEC_VERSION_MINOR > 24))
+      channels = ctx->streams[i]->codecpar->ch_layout.nb_channels;
+#else
       channels = ctx->streams[i]->codecpar->channels;
+#endif
       switch (codec_type)
 	{
 	  case AVMEDIA_TYPE_VIDEO:
