@@ -44,9 +44,13 @@
 #ifdef HAVE_UUID
 #include <uuid/uuid.h>
 #endif
+#ifdef HAVE_PTHREAD_NP_H
+# include <pthread_np.h>
+#endif
 
 #include <netdb.h>
 #include <arpa/inet.h>
+#include <ifaddrs.h> // getifaddrs
 
 #include <unistr.h>
 #include <uniconv.h>
@@ -70,6 +74,9 @@ static char *buildopts[] =
 #else
     "Without Spotify",
 #endif
+#ifdef SPOTIFY_LIBRESPOTC
+    "librespot-c",
+#endif
 #ifdef LASTFM
     "LastFM",
 #else
@@ -154,13 +161,13 @@ net_peer_address_is_trusted(const char *addr)
 int
 net_address_get(char *addr, size_t addr_len, union net_sockaddr *naddr)
 {
-  const char *s;
+  const char *s = NULL;
 
   memset(addr, 0, addr_len); // Just in case caller doesn't check for errors
 
   if (naddr->sa.sa_family == AF_INET6)
      s = inet_ntop(AF_INET6, &naddr->sin6.sin6_addr, addr, addr_len);
-  else
+  else if (naddr->sa.sa_family == AF_INET)
      s = inet_ntop(AF_INET, &naddr->sin.sin_addr, addr, addr_len);
 
   if (!s)
@@ -174,12 +181,45 @@ net_port_get(short unsigned *port, union net_sockaddr *naddr)
 {
   if (naddr->sa.sa_family == AF_INET6)
      *port = ntohs(naddr->sin6.sin6_port);
-  else
+  else if (naddr->sa.sa_family == AF_INET)
      *port = ntohs(naddr->sin.sin_port);
+  else
+    return -1;
 
   return 0;
 }
 
+int
+net_if_get(char *ifname, size_t ifname_len, const char *addr)
+{
+  struct ifaddrs *ifaddrs;
+  struct ifaddrs *iap;
+  char s[64];
+
+  memset(ifname, 0, ifname_len);
+
+  getifaddrs(&ifaddrs);
+
+  for (iap = ifaddrs; iap; iap = iap->ifa_next)
+    {
+      if (!iap->ifa_addr)
+	continue;
+
+      if (net_address_get(s, sizeof(s), (union net_sockaddr *)iap->ifa_addr) < 0)
+	continue;
+
+      if (strcmp(s, addr) != 0)
+	continue;
+
+      snprintf(ifname, ifname_len, "%s", iap->ifa_name);
+      break;
+    }
+
+  freeifaddrs(ifaddrs);
+
+  return (ifname[0] != 0) ? 0 : -1;
+}
+
 int
 net_connect(const char *addr, unsigned short port, int type, const char *log_service_name)
 {
@@ -242,12 +282,14 @@ net_bind(short unsigned *port, int type, const char *log_service_name)
   struct addrinfo hints = { 0 };
   struct addrinfo *servinfo;
   struct addrinfo *ptr;
+  union net_sockaddr naddr = { 0 };
+  socklen_t naddr_len = sizeof(naddr);
   const char *cfgaddr;
   char addr[INET6_ADDRSTRLEN];
   char strport[8];
   int yes = 1;
   int no = 0;
-  int fd;
+  int fd = -1;
   int ret;
 
   cfgaddr = cfg_getstr(cfg_getsec(cfg, "general"), "bind_address");
@@ -264,7 +306,7 @@ net_bind(short unsigned *port, int type, const char *log_service_name)
       return -1;
     }
 
-  for (ptr = servinfo, fd = -1; ptr != NULL; ptr = ptr->ai_next)
+  for (ptr = servinfo; ptr != NULL; ptr = ptr->ai_next)
     {
       if (fd >= 0)
 	close(fd);
@@ -305,23 +347,30 @@ net_bind(short unsigned *port, int type, const char *log_service_name)
       goto error;
     }
 
-  // Get the port that was assigned
-  ret = getsockname(fd, ptr->ai_addr, &ptr->ai_addrlen);
+  // Get our address (as string) and the port that was assigned (necessary when
+  // caller didn't specify a port)
+  ret = getsockname(fd, &naddr.sa, &naddr_len);
   if (ret < 0)
     {
-      DPRINTF(E_LOG, L_MISC, "Could not find address of service '%s': %s\n", log_service_name, strerror(errno));
+      DPRINTF(E_LOG, L_MISC, "Error finding address of service '%s': %s\n", log_service_name, strerror(errno));
+      goto error;
+    }
+  else if (naddr_len > sizeof(naddr))
+    {
+      DPRINTF(E_LOG, L_MISC, "Unexpected address length of service '%s'\n", log_service_name);
       goto error;
     }
 
-  net_port_get(port, (union net_sockaddr *)ptr->ai_addr);
-  net_address_get(addr, sizeof(addr), (union net_sockaddr *)ptr->ai_addr);
+  net_port_get(port, &naddr);
+  net_address_get(addr, sizeof(addr), &naddr);
 
   DPRINTF(E_DBG, L_MISC, "Service '%s' bound to %s, port %hu, socket %d\n", log_service_name, addr, *port, fd);
 
   return fd;
 
  error:
-  close(fd);
+  if (fd >= 0)
+    close(fd);
   return -1;
 }
 
@@ -333,8 +382,11 @@ net_evhttp_bind(struct evhttp *evhttp, short unsigned port, const char *log_serv
   int ret;
 
   bind_address = cfg_getstr(cfg_getsec(cfg, "general"), "bind_address");
-  if (bind_address)
-    evhttp_bind_socket(evhttp, bind_address, port);
+
+  // Normally comply with config, except for "::" where we want to listen on
+  // both ipv4 and ipv6 (as the comment in the config file says)
+  if (bind_address && strcmp(bind_address, "::") != 0)
+    return evhttp_bind_socket(evhttp, bind_address, port);
 
   // For Linux, we could just do evhttp_bind_socket() for "::", and both the
   // ipv4 and v6 port would be bound. However, for bsd it seems it is necessary
@@ -378,7 +430,11 @@ safe_atoi32(const char *str, int32_t *val)
   char *end;
   long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_atoi32 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtol(str, &end, 10);
@@ -387,21 +443,18 @@ safe_atoi32(const char *str, int32_t *val)
       || ((errno != 0) && (intval == 0)))
     {
       DPRINTF(E_DBG, L_MISC, "Invalid integer in string (%s): %s\n", str, strerror(errno));
-
       return -1;
     }
 
   if (end == str)
     {
       DPRINTF(E_DBG, L_MISC, "No integer found in string (%s)\n", str);
-
       return -1;
     }
 
   if (intval > INT32_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -416,7 +469,11 @@ safe_atou32(const char *str, uint32_t *val)
   char *end;
   unsigned long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_atou32 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtoul(str, &end, 10);
@@ -425,21 +482,18 @@ safe_atou32(const char *str, uint32_t *val)
       || ((errno != 0) && (intval == 0)))
     {
       DPRINTF(E_DBG, L_MISC, "Invalid integer in string (%s): %s\n", str, strerror(errno));
-
       return -1;
     }
 
   if (end == str)
     {
       DPRINTF(E_DBG, L_MISC, "No integer found in string (%s)\n", str);
-
       return -1;
     }
 
   if (intval > UINT32_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -454,7 +508,11 @@ safe_hextou32(const char *str, uint32_t *val)
   char *end;
   unsigned long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_hextou32 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtoul(str, &end, 16);
@@ -463,21 +521,18 @@ safe_hextou32(const char *str, uint32_t *val)
       || ((errno != 0) && (intval == 0)))
     {
       DPRINTF(E_DBG, L_MISC, "Invalid integer in string (%s): %s\n", str, strerror(errno));
-
       return -1;
     }
 
   if (end == str)
     {
       DPRINTF(E_DBG, L_MISC, "No integer found in string (%s)\n", str);
-
       return -1;
     }
 
   if (intval > UINT32_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -492,7 +547,11 @@ safe_atoi64(const char *str, int64_t *val)
   char *end;
   long long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_atoi64 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtoll(str, &end, 10);
@@ -501,21 +560,18 @@ safe_atoi64(const char *str, int64_t *val)
       || ((errno != 0) && (intval == 0)))
     {
       DPRINTF(E_DBG, L_MISC, "Invalid integer in string (%s): %s\n", str, strerror(errno));
-
       return -1;
     }
 
   if (end == str)
     {
       DPRINTF(E_DBG, L_MISC, "No integer found in string (%s)\n", str);
-
       return -1;
     }
 
   if (intval > INT64_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -530,7 +586,11 @@ safe_atou64(const char *str, uint64_t *val)
   char *end;
   unsigned long long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_atou64 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtoull(str, &end, 10);
@@ -553,7 +613,6 @@ safe_atou64(const char *str, uint64_t *val)
   if (intval > UINT64_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -568,7 +627,11 @@ safe_hextou64(const char *str, uint64_t *val)
   char *end;
   unsigned long long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_hextou64 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtoull(str, &end, 16);
@@ -577,21 +640,18 @@ safe_hextou64(const char *str, uint64_t *val)
       || ((errno != 0) && (intval == 0)))
     {
       DPRINTF(E_DBG, L_MISC, "Invalid integer in string (%s): %s\n", str, strerror(errno));
-
       return -1;
     }
 
   if (end == str)
     {
       DPRINTF(E_DBG, L_MISC, "No integer found in string (%s)\n", str);
-
       return -1;
     }
 
   if (intval > UINT64_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -685,13 +745,15 @@ safe_snreplace(char *s, size_t sz, const char *pattern, const char *replacement)
       if (dst + num > s + sz)
 	return -1; // Not enough room
 
-      // Shift everything after the pattern to the right, use memmove since
-      // there might be an overlap
+      // Move everything after the pattern, use memmove since there might be an overlap
       memmove(dst, src, num);
 
       // Write replacement, no null terminater
       memcpy(ptr, replacement, r_len);
 
+      // String now has a new length
+      s_len += r_len - p_len;
+
       // Advance ptr to avoid infinite looping
       ptr = dst;
     }
@@ -1539,6 +1601,16 @@ mutex_init(pthread_mutex_t *mutex)
   return err;
 }
 
+void
+thread_setname(pthread_t thread, const char *name)
+{
+#if defined(HAVE_PTHREAD_SETNAME_NP)
+  pthread_setname_np(thread, name);
+#elif defined(HAVE_PTHREAD_SET_NAME_NP)
+  pthread_set_name_np(thread, name);
+#endif
+}
+
 #ifdef HAVE_UUID
 void
 uuid_make(char *str)
