@@ -59,9 +59,6 @@
 #include <errno.h>
 #include <time.h>
 #include <pthread.h>
-#ifdef HAVE_PTHREAD_NP_H
-# include <pthread_np.h>
-#endif
 
 #ifdef HAVE_TIMERFD
 # include <sys/timerfd.h>
@@ -1509,7 +1506,7 @@ device_streaming_cb(struct output_device *device, enum output_device_state statu
   // We don't do this in the other cb's because they are triggered by a command
   // and thus the update should be done as part of the command completion (which
   // can better determine which type of listener event to use)
-  status_update(player_state, LISTENER_SPEAKER);
+  status_update(player_state, LISTENER_SPEAKER | LISTENER_VOLUME);
 }
 
 static void
@@ -2712,6 +2709,10 @@ speaker_generic_bh(void *arg, int *retval)
  * device-busy=0             | Playback does not resume, device not grayed    | Playback resumes on device, device not grayed
  * (same if vice versa, ie busy=1 first)
  *
+ * Special cases:
+ *  a) Denon Home 150: User deselects speaker, we send TEARDOWN, speaker then
+ *     sends prevent-playback=1 and then shortly after prevent-play-playback=0.
+ *     In this case we never want to resume playback.
  */
 static enum command_state
 speaker_prevent_playback_set(void *arg, int *retval)
@@ -2723,17 +2724,21 @@ speaker_prevent_playback_set(void *arg, int *retval)
   if (!device)
     return COMMAND_END;
 
-  device->prevent_playback = param->prevent_playback;
+  DPRINTF(E_DBG, L_PLAYER, "Speaker prevent playback change %u -> %u: '%s' (id=%" PRIu64 ")\n",
+    device->prevent_playback, param->prevent_playback, device->name, device->id);
 
-  DPRINTF(E_DBG, L_PLAYER, "Speaker prevent playback: '%s' (id=%" PRIu64 ")\n", device->name, device->id);
-
-  if (device->prevent_playback)
+  if (param->prevent_playback && device->state == OUTPUT_STATE_STOPPED)
+    *retval = -1; // Means we won't set device->prevent_playback below, so that special case a) will work
+  else if (param->prevent_playback)
     *retval = outputs_device_stop(device, device_shutdown_cb);
-  else if (!device->busy)
+  else if (!device->busy && device->prevent_playback) // Only start if previously prevented
     *retval = outputs_device_start(device, device_activate_cb, PLAYER_ONLY_PROBE);
   else
     *retval = 0;
 
+  if (*retval >= 0)
+    device->prevent_playback = param->prevent_playback;
+
   if (*retval > 0)
     return COMMAND_PENDING; // async
 
@@ -2745,10 +2750,10 @@ speaker_prevent_playback_set_bh(void *arg, int *retval)
 {
   struct speaker_attr_param *param = arg;
 
-  if (outputs_sessions_count() == 0)
+  if (player_state == PLAY_PLAYING && outputs_sessions_count() == 0)
     {
-      DPRINTF(E_INFO, L_PLAYER, "Ending playback, speaker (id=%" PRIu64 ") set 'busy' or 'prevent-playback' flag\n", param->spk_id);
-      pb_abort(); // TODO Would be better for the user if we paused, but we don't have a handy function for that
+      DPRINTF(E_INFO, L_PLAYER, "Suspending playback, speaker (id=%" PRIu64 ") set 'busy' or 'prevent-playback' flag\n", param->spk_id);
+      pb_suspend(); // Don't want to use pb_abort here, since that may clear the queue
     }
   else
     status_update(player_state, LISTENER_SPEAKER | LISTENER_VOLUME);
@@ -2767,17 +2772,21 @@ speaker_busy_set(void *arg, int *retval)
   if (!device)
     return COMMAND_END;
 
-  device->busy = param->busy;
+  DPRINTF(E_DBG, L_PLAYER, "Speaker busy change %u -> %u: '%s' (id=%" PRIu64 ")\n",
+    device->busy, param->busy, device->name, device->id);
 
-  DPRINTF(E_DBG, L_PLAYER, "Speaker busy: '%s' (id=%" PRIu64 ")\n", device->name, device->id);
-
-  if (device->busy)
+  if (param->busy && device->state == OUTPUT_STATE_STOPPED)
+    *retval = -1; // Means we won't set device->busy below, so that special case a) will work
+  else if (param->busy)
     *retval = outputs_device_stop(device, device_shutdown_cb);
-  else if (!device->prevent_playback)
+  else if (!device->prevent_playback && device->busy) // Only start if previously busy
     *retval = outputs_device_start(device, device_activate_cb, PLAYER_ONLY_PROBE);
   else
     *retval = 0;
 
+  if (*retval >= 0)
+    device->busy = param->busy;
+
   if (*retval > 0)
     return COMMAND_PENDING; // async
 
@@ -2938,9 +2947,8 @@ volume_setabs_speaker(void *arg, int *retval)
   return COMMAND_END;
 }
 
-// Just updates internal volume params (does not make actual requests to the speaker)
 static enum command_state
-volume_update_speaker(void *arg, int *retval)
+volume_setraw_speaker(void *arg, int *retval)
 {
   struct speaker_attr_param *vol_param = arg;
   struct output_device *device;
@@ -2964,7 +2972,11 @@ volume_update_speaker(void *arg, int *retval)
 
   outputs_device_volume_register(device, volume, -1);
 
-  *retval = 0;
+  *retval = outputs_device_volume_set(device, device_volume_cb);
+
+  if (*retval > 0)
+    return COMMAND_PENDING; // async
+
   return COMMAND_END;
 }
 
@@ -2975,10 +2987,14 @@ volume_generic_bh(void *arg, int *retval)
   return COMMAND_END;
 }
 
+static enum command_state
+consume_set(void *arg, int *retval);
+
 static enum command_state
 repeat_set(void *arg, int *retval)
 {
   enum repeat_mode *mode = arg;
+  union player_arg consume_arg;
 
   if (*mode == repeat)
     {
@@ -3003,6 +3019,13 @@ repeat_set(void *arg, int *retval)
   // Persist
   SETTINGS_SETINT(player_settings_category, PLAYER_SETTINGS_MODE_REPEAT, repeat);
 
+  if (repeat == REPEAT_ALL || repeat == REPEAT_SONG)
+    {
+      // Activating repeat requires repeat consume mode to be off
+      consume_arg.intval = 0;
+      consume_set(&consume_arg, retval);
+    }
+
   *retval = 0;
   return COMMAND_END;
 }
@@ -3046,6 +3069,7 @@ shuffle_set(void *arg, int *retval)
 static enum command_state
 consume_set(void *arg, int *retval)
 {
+  enum repeat_mode repeat_mode;
   union player_arg *cmdarg = arg;
 
   consume = cmdarg->intval;
@@ -3053,6 +3077,13 @@ consume_set(void *arg, int *retval)
   // Persist
   SETTINGS_SETBOOL(player_settings_category, PLAYER_SETTINGS_MODE_CONSUME, consume);
 
+  if (consume)
+    {
+      // Activating cosume mode requires repeat mode to be off
+      repeat_mode = REPEAT_OFF;
+      repeat_set(&repeat_mode, retval);
+    }
+
   *retval = 0;
   return COMMAND_END;
 }
@@ -3453,7 +3484,7 @@ player_volume_setabs_speaker(uint64_t id, int vol)
 }
 
 int
-player_volume_update_speaker(uint64_t id, const char *volstr)
+player_volume_setraw_speaker(uint64_t id, const char *volstr)
 {
   struct speaker_attr_param vol_param;
   int ret;
@@ -3461,7 +3492,7 @@ player_volume_update_speaker(uint64_t id, const char *volstr)
   vol_param.spk_id = id;
   vol_param.volstr  = volstr;
 
-  ret = commands_exec_sync(cmdbase, volume_update_speaker, volume_generic_bh, &vol_param);
+  ret = commands_exec_sync(cmdbase, volume_setraw_speaker, volume_generic_bh, &vol_param);
 
   return ret;
 }
@@ -3631,7 +3662,13 @@ player_init(void)
   int ret;
 
   speaker_autoselect = cfg_getbool(cfg_getsec(cfg, "general"), "speaker_autoselect");
-  clear_queue_on_stop_disabled = cfg_getbool(cfg_getsec(cfg, "mpd"), "clear_queue_on_stop_disable");
+  clear_queue_on_stop_disabled = cfg_getbool(cfg_getsec(cfg, "library"), "clear_queue_on_stop_disable");
+
+  /* Handle deprecated config options, note that this is also in library.c */
+  if (0 < cfg_opt_size(cfg_getopt(cfg_getsec(cfg, "mpd"), "clear_queue_on_stop_disable")))
+    {
+      clear_queue_on_stop_disabled = cfg_getbool(cfg_getsec(cfg, "mpd"), "clear_queue_on_stop_disable");
+    }
 
   CHECK_NULL(L_PLAYER, player_settings_category = settings_category_get("player"));
   ret = SETTINGS_GETINT(player_settings_category, PLAYER_SETTINGS_MODE_REPEAT);
@@ -3705,11 +3742,8 @@ player_init(void)
       DPRINTF(E_FATAL, L_PLAYER, "Could not spawn player thread: %s\n", strerror(errno));
       goto error_input_deinit;
     }
-#if defined(HAVE_PTHREAD_SETNAME_NP)
-  pthread_setname_np(tid_player, "player");
-#elif defined(HAVE_PTHREAD_SET_NAME_NP)
-  pthread_set_name_np(tid_player, "player");
-#endif
+
+  thread_setname(tid_player, "player");
 
   return 0;
 
